#!/usr/bin/env bash

set -o errexit
set -o nounset
set -o xtrace

# Prepare environment
export DEBIAN_FRONTEND=noninteractive
GATUS_BUILD_DIR=/opt/gatus
GATUS_TLS_DIR=/etc/nginx/tls
GATUS_CONFIG_PATH=/etc/gatus.yaml

mkdir -p $GATUS_BUILD_DIR
mkdir -p $GATUS_TLS_DIR

# Prepare Gatus user/group
useradd -M -r gatus

# Update system
apt-get install -y apt-utils
apt-get update && apt-get upgrade -y
apt-get install -y curl nginx nfs-common fcgiwrap socat

# Install Go (https://go.dev/doc/install)
cd $GATUS_BUILD_DIR
curl -Lo go.tar.gz https://go.dev/dl/go1.20.7.linux-armv6l.tar.gz
tar -C /usr/local -xf go.tar.gz
export PATH=$PATH:/usr/local/go/bin

# Download Gatus
GATUS_VERSION=5.10.0
curl -Lo gatus.tar.gz https://github.com/TwiN/gatus/archive/refs/tags/v$GATUS_VERSION.tar.gz
tar -xf gatus.tar.gz

# Build Gatus
cd gatus-$GATUS_VERSION
CGO_ENABLED=0 GOOS=linux go build -a -installsuffix cgo -o gatus .

# Install Gatus
mv gatus /usr/bin/gatus

# Configure Gatus (https://github.com/TwiN/gatus/tree/master#usage)
cat <<EOF > $GATUS_CONFIG_PATH
web:
  address: 127.0.0.1
endpoints:
  - name: website                 # Name of your endpoint, can be anything
    url: "https://twin.sh/health"
    interval: 5m                  # Duration to wait between every status check (default: 60s)
    conditions:
      - "[STATUS] == 200"         # Status must be 200
      - "[BODY].status == UP"     # The json path "$.status" must be equal to UP
      - "[RESPONSE_TIME] < 300"   # Response time must be under 300ms
EOF

# Create Gatus Systemd service
# https://aur.archlinux.org/cgit/aur.git/tree/gatus.service?h=gatus
cat <<EOF > /etc/systemd/system/gatus.service
[Unit]
Description=Automated service health dashboard
Wants=network-online.target
After=network-online.target

[Service]
Type=simple
User=gatus
Group=gatus
ExecStart=/usr/bin/gatus web.address
Environment=GATUS_CONFIG_PATH=$GATUS_CONFIG_PATH
Restart=on-failure
NoNewPrivileges=yes
PrivateTmp=yes
PrivateDevices=yes
DevicePolicy=closed
ReadWritePaths=/var/lib/gatus
ProtectSystem=strict
ProtectHome=read-only
ProtectControlGroups=yes
ProtectKernelModules=yes
ProtectKernelTunables=yes
RestrictAddressFamilies=AF_UNIX AF_INET AF_INET6 AF_NETLINK
RestrictNamespaces=yes
RestrictRealtime=yes
RestrictSUIDSGID=yes
MemoryDenyWriteExecute=yes
LockPersonality=yes
ProtectClock=yes
ProtectHostname=yes
ProtectKernelLogs=yes
ProtectHome=yes
CapabilityBoundingSet=CAP_NET_RAW
AmbientCapabilities=CAP_NET_RAW
UMask=0077
SystemCallFilter=~@clock
SystemCallFilter=~@cpu-emulation
SystemCallFilter=~@debug
SystemCallFilter=~@module
SystemCallFilter=~@mount
SystemCallFilter=~@obsolete
SystemCallFilter=~@privileged
SystemCallFilter=~@raw-io
SystemCallFilter=~@reboot
SystemCallFilter=~@swap

[Install]
WantedBy=multi-user.target
EOF

# Mount TLS certificate
cat <<EOF >> /etc/fstab
$GATUS_TLS_NFS_SERVER $GATUS_TLS_DIR nfs rw 0 0
EOF

# Configure Nginx reverse proxy
cat <<EOF > /etc/nginx/conf.d/gatus.conf
server {
    listen               443 ssl;

    ssl_certificate      $GATUS_TLS_DIR/live/up.in0rdr.ch/fullchain.pem;
    ssl_certificate_key  $GATUS_TLS_DIR/live/up.in0rdr.ch/privkey.pem;

    location / {
        proxy_pass       http://127.0.0.1:8080;
        proxy_set_header Host \$host;
        proxy_set_header X-Real-IP \$remote_addr;
        proxy_set_header X-Forwarded-For \$proxy_add_x_forwarded_for;
        proxy_set_header X-Forwarded-Proto \$scheme;
    }
}

server {
    listen 127.0.0.1:9090;

    # expose the collectd metrics
    location /collectd {
        include fastcgi_params;
        fastcgi_param SCRIPT_FILENAME /usr/local/bin/collectd.sh;
        fastcgi_param QUERY_STRING \$query_string;
        fastcgi_pass unix:/run/fcgiwrap.socket;
    }
}
EOF

# Allow Gatus to read config file
chown gatus: $GATUS_CONFIG_PATH

# Enable Gatus and Nginx
systemctl enable gatus
systemctl enable nginx

# Ensure tls on nfs before nginx starts
mkdir -p /etc/systemd/system/nginx.service.d
cat <<EOF > /etc/systemd/system/nginx.service.d/override.conf
[Service]
ExecStartPre=
ExecStartPre=/usr/bin/sh -c 'sleep 5 && mount -a'
ExecStartPre=/usr/sbin/nginx -t -q -g 'daemon on; master_process on;'
EOF

# Add collectd script to return stats as HTTP response
cat <<'EOF' > /usr/local/bin/collectd.sh
#!/usr/bin/env bash
#
# Depends:
# - fcgiwrap
# - socat
#
# https://landchad.net/cgi

COLLECTD_SOCK=/collectd/query.sock
#echo "LISTVAL" | socat - UNIX-CLIENT:$COLLECTD_SOCK

# We need the script primarily, because the collectd unix socket plugin does
# not return a proper http response. This line here fixes that behavior so it
# is a proper http response. Nginx will refuse to return the response from the
# unix upstream otherwise. Unfortunately, the Nginx add_header directive would
# only add the header starting from Nginx server (to the end client).
echo "Content-type: text/json"
echo

function get_df() {
  free_e=$(echo "GETVAL $1/percent_bytes-free" | socat - UNIX-CLIENT:/$COLLECTD_SOCK | tail -1 | cut -d '=' -f2)

  # convert from scientific notation
  printf "%.0f\n" "$free_e"
}

function md_status(){
  md_status_e=$(echo "GETVAL turris/exec-$1/md_disks-degraded" | socat - UNIX-CLIENT:/$COLLECTD_SOCK | tail -1 | cut -d '=' -f2)
  # convert from scientific notation
  printf "%.0f\n" "$md_status_e"
}

df_srv=$(get_df "turris/df-srv")
df_nfs=$(get_df "turris/df-srv-nfs")
md_status=$(md_status "md127")

cat << EOF
{
  "srv_free_percent":"$df_srv",
  "nfs_free_percent":"$df_nfs",
  "md_degraded":"$md_status"
}
EOF
chmod +x /usr/local/bin/collectd.sh
